#include "tpmwrite.h"
#include "btrace.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#else
#define VERSION "unknown"
#endif

#include <stdio.h>
#include <errno.h>
#include <string.h>

typedef struct {
  int n;
  FILE *fd;
} write_point_context_t;

/* a gstack_foreach callback function */

static int write_point(void *vpoint, void *vcontext)
{
  tpm_point_t *point = vpoint;
  write_point_context_t *context = vcontext;
  const char fmt[] =
    "    CONTROLPOINT %i\n"
    "      {\n"
    "      COLORMAPFRACTION = %.6f\n"
    "      LEADRGB\n"
    "        {\n"
    "          R = %i\n"
    "          G = %i\n"
    "          B = %i\n"
    "        }\n"
    "      TRAILRGB\n"
    "        {\n"
    "          R = %i\n"
    "          G = %i\n"
    "          B = %i\n"
    "        }\n"
    "      }\n";

  fprintf(context->fd, fmt,
          context->n,
          point->fraction,
          point->lead.red,
          point->lead.green,
          point->lead.blue,
          point->trail.red,
          point->trail.green,
          point->trail.blue);

  context->n += 1;

  return 1;
}

static int write_fd(tpm_t *tpm, FILE *fd)
{
  const char
    fmt_header[] =
    "#!MC 1410\n"
    "# cptutils %s\n"
    "$!COLORMAP\n"
    "  CONTOURCOLORMAP = UserDef\n"
    "$!COLORMAPCONTROL RESETTOFACTORY\n"
    "$!COLORMAP\n"
    "  USERDEFINED\n"
    "    {\n"
    "    NUMCONTROLPOINTS = %zi\n",
    fmt_footer[] =
    "    }\n";

  fprintf(fd, fmt_header, VERSION, tpm_npoint(tpm));

  write_point_context_t context = { .n = 1, .fd = fd };
  int err = tpm_each_point(tpm, write_point, &context);

  fprintf(fd, fmt_footer);

  return err;
}

extern int tpm_write(tpm_t *tpm, const char *path)
{
  FILE *fd;
  int err = 0;

  if (path)
    {
      if ((fd = fopen(path, "w")) != NULL)
        {
          err = write_fd(tpm, fd);
          fclose(fd);
        }
      else
        {
          btrace("failed to open %s (%s)", path, strerror(errno));
          err++;
        }
    }
  else
    err = write_fd(tpm, stdout);

  return err;
}
